// Container module.
module SoS_Explorer.Objective

    // Calculate the objective's value for a given architecture.
    let Objective_Affordability(characteristics : float[,], capabilities : bool[,],
                                feasibleInterfaces : bool[,],directedInterfaces : bool,
                                negotiatedTime : float, negotiatedCost : float,
                                architecture : bool[]) =

        // Bounds
        let lowerBound =   0.0
        let upperBound = 100.0
        let bias       =   0.0

        // Number of elements
        let numSystems         = Array2D.length1 characteristics
        let numCharacteristics = Array2D.length2 characteristics
        let numCapabilities    = Array2D.length2 capabilities

        // System indices
        let sys_ReportA      = 0
        let sys_ReportB      = 1
        let sys_ReportC      = 2
        let sys_ReportD      = 3
        let sys_VisualA      = 4
        let sys_VisualB      = 5
        let sys_VisualC      = 6
        let sys_VisualD      = 7
        let sys_VisualE      = 8
        let sys_ClusterA     = 9
        let sys_ClusterB     = 10
        let sys_ClusterC     = 11
        let sys_PMA          = 12
        let sys_PMB          = 13
        let sys_PMC          = 14
        let sys_InhouseA     = 15
        let sys_InhouseB     = 16
        let sys_InhouseC     = 17
        let sys_InhouseD     = 18
        let sys_InhouseE     = 19
        let sys_InhouseF     = 20
        let sys_ApplicationA = 20
        let sys_ApplicationB = 21
        let sys_ApplicationC = 22

        // Characteristic indices
        let char_Language      = 0
        let char_Cost          = 1
        let char_Performance   = 2
        let char_Scalability   = 3
        let char_HasRG         = 4
        let char_HasVisual     = 5
        let char_HasClustering = 6
        let char_HasPM         = 7
        let char_HasApp        = 8

        // Capability indices
        let cap_OOXML            = 0
        let cap_PDF              = 1
        let cap_Graph            = 2
        let cap_Flow             = 3
        let cap_Multidimensional = 4
        let cap_Geospatial       = 5
        let cap_BIRCH            = 6
        let cap_DBSCAN           = 7
        let cap_SVM              = 8
        let cap_GLM              = 9
        let cap_DBN              = 10
        let cap_Application      = 11

        // Define hasSystem() function
        // Returns whether the given system is specified in the chromosome
        let hasSystem i =
            // Return whether system is specified
            architecture.[i]

        // Define hasInterface() function
        // Returns whether the given interface is specified in the chromosome
        let hasInterface i j =
            // If same system
            if i = j then
                false
            else
                // Index of interface in chromosome
                let jj = if j < i then j + 1 else j
                let k = 
                    if directedInterfaces then
                        (numSystems - 1) * (i + 1) + jj
                    else
                        let m = (min i j) + 1
                        m * numSystems - m * (m - 1) / 2 + (abs(i - j)) - 1
                // Return whether interface is specified
                architecture.[k]

        // Define beta (inter-language cost matrix)
        let beta = array2D [ [0.01; 0.02; 0.04];
                             [0.02; 0.01; 0.04];
                             [0.04; 0.04; 0.01] ]
        
        // Calculate cost
        let mutable maxCost = 0.0
        let mutable totalCost = negotiatedCost
        for i = 0 to numSystems - 1 do
            maxCost <- maxCost + characteristics.[i, char_Cost]
            if hasSystem i then
                totalCost <- totalCost + characteristics.[i, char_Cost]
            let iLang = (int characteristics.[i, char_Language]) - 1
            for j = 0 to numSystems - 1 do
                if i <> j then
                    let jLang = (int characteristics.[j, char_Language]) - 1
                    maxCost <- maxCost +
                               beta.[iLang, jLang] * characteristics.[j, char_Cost]
                    if hasInterface i j then
                        totalCost <- totalCost +
                                     beta.[iLang, jLang] * characteristics.[j, char_Cost]
        let cost = lowerBound + (upperBound - lowerBound) * totalCost / maxCost
        let affordability = lowerBound + upperBound - cost

        // Return bounded results
        max lowerBound (min upperBound (affordability + bias))
