% Constrain given architecture via chromosome fixing.

function fixed = FixChromosome( ...
    characteristics,capabilities,feasibleInterfaces, ...
    directedInterfaces,architecture)

% Number of elements
numSystems         = size(characteristics,1);
numCharacteristics = size(characteristics,2);
numCapabilities    = size(capabilities,2);

% System indices
sys_ReportA      = 1;
sys_ReportB      = 2;
sys_ReportC      = 3;
sys_ReportD      = 4;
sys_VisualA      = 5;
sys_VisualB      = 6;
sys_VisualC      = 7;
sys_VisualD      = 8;
sys_VisualE      = 9;
sys_ClusterA     = 10;
sys_ClusterB     = 11;
sys_ClusterC     = 12;
sys_PMA          = 13;
sys_PMB          = 14;
sys_PMC          = 15;
sys_InhouseA     = 16;
sys_InhouseB     = 17;
sys_InhouseC     = 18;
sys_InhouseD     = 19;
sys_InhouseE     = 20;
sys_InhouseF     = 21;
sys_ApplicationA = 22;
sys_ApplicationB = 23;
sys_ApplicationC = 24;

% Characteristic indices
char_Language      = 1;
char_Cost          = 2;
char_Performance   = 3;
char_Scalability   = 4;
char_HasRG         = 5;
char_HasVisual     = 6;
char_HasClustering = 7;
char_HasPM         = 8;
char_HasApp        = 9;

% Capability indices
cap_OOXML            = 1;
cap_PDF              = 2;
cap_Graph            = 3;
cap_Flow             = 4;
cap_Multidimensional = 5;
cap_Geospatial       = 6;
cap_BIRCH            = 7;
cap_DBSCAN           = 8;
cap_SVM              = 9;
cap_GLM              = 10;
cap_DBN              = 11;
cap_Application      = 12;

% Define HasSystem() function
HasSystem = @(arch,i) HasSystemF(arch,i);

% Define HasInterface() function
HasInterface = @(arch,i,j) HasInterfaceF( ...
    arch,numSystems,directedInterfaces,i,j);

% Define HasPath() function
HasPath = @(arch,i,j) HasPathF( ...
    arch,numSystems,directedInterfaces,i,j);

% Define SetSystem() function
SetSystem = @(arch,i,val) SetSystemF(arch,i,val);

% Define SetInterface() function
SetInterface = @(arch,i,j,val) SetInterfaceF( ...
    arch,numSystems,directedInterfaces,i,j,val);

% Fixed chromosome
fixed = architecture;

% Define gamma (required paths matrix)
gamma = [false false true  true  true; ...
         false false true  true  true; ...
         true  true  false false true; ...
         true  true  false false true; ...
         true  true  true  true  true];

% Identify and add missing capabilities
for i = 1:numCapabilities
    j = 1;
    k = 0;
    hasCapability = false;
    while ((~hasCapability) && (j<=numSystems))
        if (capabilities(j,i))
            if (HasSystem(fixed,j))
                hasCapability = true;
            else
                k = j;
            end
        end
        j = j+1;
    end
    if ((~hasCapability) && (k>0))
        fixed = SetSystem(fixed,k,true);
    end
end

% Remove infeasible interfaces from architecture
for i = 1:numSystems
    for j = 1:numSystems
        % If different systems
        if (i ~= j)
            % If interface is specified
            if (HasInterface(fixed,i,j))
                % If infeasible, remove
                if (~(HasSystem(fixed,i) && HasSystem(fixed,j) && ...
                      feasibleInterfaces(i,j)))
                    fixed = SetInterface(fixed,i,j,false);
                end
            end
        end
    end
end

% Add required interfaces to architecture
for i = 1:numSystems
    if (HasSystem(fixed,i))
        for m = char_HasRG:char_HasApp
            if (abs(characteristics(i,m)) > 0.01)
                for n = char_HasRG:char_HasApp
                    if ((m ~= n) && gamma(m-char_HasRG+1,n-char_HasRG+1))
                        for j = 1:numSystems
                            if ((i ~= j) && HasSystem(fixed,j) && (abs(characteristics(j,n)) > 0.01))
                                if (~HasPath(fixed,i,j))
                                    fixed = SetInterface(fixed,i,j,true);
                                end
                            end
                        end
                    end
                end
            end
        end
    end
end
