% Calculate the objective's value for a given architecture.

function objective = Objective_Affordability( ...
    characteristics,capabilities,feasibleInterfaces, ...
    directedInterfaces,negotiatedTime,negotiatedCost,architecture)

% Bounds
lowerBound =   0;
upperBound = 100;
bias       =   0;

% Number of elements
numSystems         = size(characteristics,1);
numCharacteristics = size(characteristics,2);
numCapabilities    = size(capabilities,2);

% System indices
sys_ReportA      = 1;
sys_ReportB      = 2;
sys_ReportC      = 3;
sys_ReportD      = 4;
sys_VisualA      = 5;
sys_VisualB      = 6;
sys_VisualC      = 7;
sys_VisualD      = 8;
sys_VisualE      = 9;
sys_ClusterA     = 10;
sys_ClusterB     = 11;
sys_ClusterC     = 12;
sys_PMA          = 13;
sys_PMB          = 14;
sys_PMC          = 15;
sys_InhouseA     = 16;
sys_InhouseB     = 17;
sys_InhouseC     = 18;
sys_InhouseD     = 19;
sys_InhouseE     = 20;
sys_InhouseF     = 21;
sys_ApplicationA = 22;
sys_ApplicationB = 23;
sys_ApplicationC = 24;

% Characteristic indices
char_Language      = 1;
char_Cost          = 2;
char_Performance   = 3;
char_Scalability   = 4;
char_HasRG         = 5;
char_HasVisual     = 6;
char_HasClustering = 7;
char_HasPM         = 8;
char_HasApp        = 9;

% Capability indices
cap_OOXML            = 1;
cap_PDF              = 2;
cap_Graph            = 3;
cap_Flow             = 4;
cap_Multidimensional = 5;
cap_Geospatial       = 6;
cap_BIRCH            = 7;
cap_DBSCAN           = 8;
cap_SVM              = 9;
cap_GLM              = 10;
cap_DBN              = 11;
cap_Application      = 12;


% Define HasSystem() function
HasSystem = @(i) HasSystemF(architecture,i);

% Define HasInterface() function
HasInterface = @(i,j) HasInterfaceF( ...
    architecture,numSystems,directedInterfaces,i,j);

% Define beta (inter-language cost matrix)
beta = [0.01 0.02 0.04; ...
        0.02 0.01 0.04; ...
        0.04 0.04 0.01];
        
% Calculate cost
maxCost = 0.0;
totalCost = negotiatedCost;
for i = 1:numSystems
    maxCost = maxCost+characteristics(i,char_Cost);
    if (HasSystem(i))
        totalCost = totalCost+characteristics(i,char_Cost);
    end
    iLang = characteristics(i,char_Language);
    for j = 1:numSystems
        if (i ~= j)
            jLang = characteristics(j,char_Language);
            maxCost = maxCost+beta(iLang,jLang)*characteristics(j,char_Cost);
            if (HasInterface(i,j))
                totalCost = totalCost+beta(iLang,jLang)*characteristics(j,char_Cost);
            end
        end
    end
end
cost = lowerBound+(upperBound-lowerBound)*totalCost/maxCost;
affordability = lowerBound+upperBound-cost;

% Return bounded results
objective = max(lowerBound,min(upperBound,affordability+bias));
