% Calculate the objective's value for a given architecture.

function objective = Objective_Flexibility( ...
    characteristics,capabilities,feasibleInterfaces, ...
    directedInterfaces,negotiatedTime,negotiatedCost,architecture)

% Bounds
lowerBound =   0;
upperBound = 100;
bias       =   0;

% Number of elements
numSystems         = size(characteristics,1);
numCharacteristics = size(characteristics,2);
numCapabilities    = size(capabilities,2);

% System indices
sys_ReportA      = 1;
sys_ReportB      = 2;
sys_ReportC      = 3;
sys_ReportD      = 4;
sys_VisualA      = 5;
sys_VisualB      = 6;
sys_VisualC      = 7;
sys_VisualD      = 8;
sys_VisualE      = 9;
sys_ClusterA     = 10;
sys_ClusterB     = 11;
sys_ClusterC     = 12;
sys_PMA          = 13;
sys_PMB          = 14;
sys_PMC          = 15;
sys_InhouseA     = 16;
sys_InhouseB     = 17;
sys_InhouseC     = 18;
sys_InhouseD     = 19;
sys_InhouseE     = 20;
sys_InhouseF     = 21;
sys_ApplicationA = 22;
sys_ApplicationB = 23;
sys_ApplicationC = 24;

% Characteristic indices
char_Language      = 1;
char_Cost          = 2;
char_Performance   = 3;
char_Scalability   = 4;
char_HasRG         = 5;
char_HasVisual     = 6;
char_HasClustering = 7;
char_HasPM         = 8;
char_HasApp        = 9;

% Capability indices
cap_OOXML            = 1;
cap_PDF              = 2;
cap_Graph            = 3;
cap_Flow             = 4;
cap_Multidimensional = 5;
cap_Geospatial       = 6;
cap_BIRCH            = 7;
cap_DBSCAN           = 8;
cap_SVM              = 9;
cap_GLM              = 10;
cap_DBN              = 11;
cap_Application      = 12;


% Define HasSystem() function
HasSystem = @(i) HasSystemF(architecture,i);

% Define HasInterface() function
HasInterface = @(i,j) HasInterfaceF( ...
    architecture,numSystems,directedInterfaces,i,j);

% Calculate flexibility
maxFlex = 0;
totalFlex = 0;
for i = 1:numCapabilities
    foundCapability = false;
    presentCapability = false;
    for j = 1:numSystems
        if (capabilities(j,i))
            if (foundCapability)
                maxFlex = maxFlex+1;
            else
                foundCapability = true;
            end
            if (HasSystem(j))
                if (presentCapability)
                    totalFlex = totalFlex+1;
                else
                    presentCapability = true;
                end
            end
        end
    end
end
flex = lowerBound+(upperBound-lowerBound)*totalFlex/maxFlex;

% Return bounded results
objective = max(lowerBound,min(upperBound,flex+bias));
