# Container class.
class Objective:

    # Class constructor
    def __init__(self):
        pass

    # Constrain given architecture via chromosome fixing.
    def FixChromosome(self, characteristics, capabilities,
                      feasibleInterfaces, directedInterfaces,
                      architecture):

        # Number of elements
        numSystems         = characteristics.GetLength(0)
        numCharacteristics = characteristics.GetLength(1)
        numCapabilities    = capabilities.GetLength(1)

        # System indices
        sys_ReportA      = 0
        sys_ReportB      = 1
        sys_ReportC      = 2
        sys_ReportD      = 3
        sys_VisualA      = 4
        sys_VisualB      = 5
        sys_VisualC      = 6
        sys_VisualD      = 7
        sys_VisualE      = 8
        sys_ClusterA     = 9
        sys_ClusterB     = 10
        sys_ClusterC     = 11
        sys_PMA          = 12
        sys_PMB          = 13
        sys_PMC          = 14
        sys_InhouseA     = 15
        sys_InhouseB     = 16
        sys_InhouseC     = 17
        sys_InhouseD     = 18
        sys_InhouseE     = 19
        sys_InhouseF     = 20
        sys_ApplicationA = 21
        sys_ApplicationB = 22
        sys_ApplicationC = 23

        # Characteristic indices
        char_Language      = 0
        char_Cost          = 1
        char_Performance   = 2
        char_Scalability   = 3
        char_HasRG         = 4
        char_HasVisual     = 5
        char_HasClustering = 6
        char_HasPM         = 7
        char_HasApp        = 8

        # Capability indices
        cap_OOXML            = 0
        cap_PDF              = 1
        cap_Graph            = 2
        cap_Flow             = 3
        cap_Multidimensional = 4
        cap_Geospatial       = 5
        cap_BIRCH            = 6
        cap_DBSCAN           = 7
        cap_SVM              = 8
        cap_GLM              = 9
        cap_DBN              = 10
        cap_Application      = 11

        # Define hasSystem() function
        # Returns whether the given system is specified in the chromosome
        def hasSystem(i):
            # Return whether system is specified
            return architecture[i]

        # Define hasInterface() function
        # Returns whether the given interface is specified in the chromosome
        def hasInterface(i, j):
            # If same system
            if i == j:
                return False
            else:
                # Index of interface in chromosome
                jj = j + 1 if j < i else j
                m = min(i, j) + 1
                k = (numSystems - 1) * (i + 1) + jj if directedInterfaces else \
                    m * numSystems - m * (m - 1) / 2 + abs(i - j) - 1
                # Return whether interface is specified
                return architecture[k]

        # Define setSystem() function
        # Sets the system in the chromosome to the specified value
        def setSystem(i, val):
            architecture[i] = val

        # Define setInterface() function
        # Sets the interface in the chromosome to the specified value
        def setInterface(i, j, val):
            # If not same system
            if i != j:
                # Index of interface in chromosome
                jj = j + 1 if j < i else j
                m = min(i, j) + 1
                k = (numSystems - 1) * (i + 1) + jj if directedInterfaces else \
                    m * numSystems - m * (m - 1) / 2 + abs(i - j) - 1
                architecture[k] = val

        # Define gamma (required paths matrix)
        gamma = [[False, False, True,  True,  True],
                 [False, False, True,  True,  True],
                 [True,  True,  False, False, True],
                 [True,  True,  False, False, True],
                 [True,  True,  True,  True,  True]]

        # Define has path function
        def hasPath(i, j):
            hasInterface(i, j)

        # Identify and add missing capabilities
        for i in range(0, numCapabilities):
            j = 0
            k = -1
            hasCapability = False
            while (not hasCapability) and (j < numSystems):
                if capabilities[j, i]:
                    if hasSystem(j):
                        hasCapability = True
                    else:
                        k = j
                j += 1
            if (not hasCapability) and (k > -1):
                setSystem(k, True)

        # Remove infeasible interfaces from architecture
        for i in range(0, numSystems):
            for j in range(0, numSystems):
                # If different systems
                if i != j:
                    # If interface is specified
                    if hasInterface(i, j):
                        # If infeasible, remove
                        if not (hasSystem(i) and hasSystem(j) and feasibleInterfaces[i, j]):
                            setInterface(i, j, False)

        # Add required interfaces to architecture
        for i in range(0, numSystems):
            if hasSystem(i):
                for m in range(char_HasRG, char_HasApp + 1):
                    if abs(characteristics[i, m]) > 0.01:
                        for n in range(char_HasRG, char_HasApp + 1):
                            if ((m != n) and gamma[m - char_HasRG][n - char_HasRG]):
                                for j in range(0, numSystems):
                                    if ((i != j) and hasSystem(j) and (abs(characteristics[j, n]) > 0.01)):
                                        if not (hasPath(i, j)):
                                            setInterface(i, j, True)
