# Container class.
class Objective:

    # Class constructor
    def __init__(self):
        pass

    # Calculate the objective's value for a given architecture.
    def Objective_Affordability(self, characteristics, capabilities,
                                feasibleInterfaces, directedInterfaces,
                                negotiatedTime, negotiatedCost, architecture):

        # Bounds
        lowerBound =   0.0
        upperBound = 100.0
        bias       =   0.0

        # Number of elements
        numSystems         = characteristics.GetLength(0)
        numCharacteristics = characteristics.GetLength(1)
        numCapabilities    = capabilities.GetLength(1)

        # System indices
        sys_ReportA      = 0
        sys_ReportB      = 1
        sys_ReportC      = 2
        sys_ReportD      = 3
        sys_VisualA      = 4
        sys_VisualB      = 5
        sys_VisualC      = 6
        sys_VisualD      = 7
        sys_VisualE      = 8
        sys_ClusterA     = 9
        sys_ClusterB     = 10
        sys_ClusterC     = 11
        sys_PMA          = 12
        sys_PMB          = 13
        sys_PMC          = 14
        sys_InhouseA     = 15
        sys_InhouseB     = 16
        sys_InhouseC     = 17
        sys_InhouseD     = 18
        sys_InhouseE     = 19
        sys_InhouseF     = 20
        sys_ApplicationA = 21
        sys_ApplicationB = 22
        sys_ApplicationC = 23

        # Characteristic indices
        char_Language      = 0
        char_Cost          = 1
        char_Performance   = 2
        char_Scalability   = 3
        char_HasRG         = 4
        char_HasVisual     = 5
        char_HasClustering = 6
        char_HasPM         = 7
        char_HasApp        = 8

        # Capability indices
        cap_OOXML            = 0
        cap_PDF              = 1
        cap_Graph            = 2
        cap_Flow             = 3
        cap_Multidimensional = 4
        cap_Geospatial       = 5
        cap_BIRCH            = 6
        cap_DBSCAN           = 7
        cap_SVM              = 8
        cap_GLM              = 9
        cap_DBN              = 10
        cap_Application      = 11

        # Define hasSystem() function
        # Returns whether the given system is specified in the chromosome
        def hasSystem(i):
            # Return whether system is specified
            return architecture[i]

        # Define hasInterface() function
        # Returns whether the given interface is specified in the chromosome
        def hasInterface(i, j):
            # If same system
            if i == j:
                return False
            else:
                # Index of interface in chromosome
                jj = j + 1 if j < i else j
                m = min(i, j) + 1
                k = (numSystems - 1) * (i + 1) + jj if directedInterfaces else \
                    m * numSystems - m * (m - 1) / 2 + abs(i - j) - 1
                # Return whether interface is specified
                return architecture[k]

        # Define beta (inter-language cost matrix)
        beta = [[0.01, 0.02, 0.04],
                [0.02, 0.01, 0.04],
                [0.04, 0.04, 0.01]]

        # Calculate cost
        maxCost = 0.0
        totalCost = negotiatedCost
        for i in range(0, numSystems):
            maxCost = maxCost + characteristics[i, char_Cost]
            if hasSystem(i):
                totalCost = totalCost + characteristics[i, char_Cost]
            iLang = int(characteristics[i, char_Language]) - 1
            for j in range(0, numSystems):
                if i != j:
                    jLang = int(characteristics[j, char_Language]) - 1
                    maxCost = maxCost + beta[iLang][jLang] * characteristics[j, char_Cost]
                    if hasInterface(i, j):
                        totalCost = totalCost + beta[iLang][jLang] * characteristics[j, char_Cost]
        cost = lowerBound + (upperBound - lowerBound) * totalCost / maxCost
        affordability = lowerBound + upperBound - cost

        # Return bounded results
        return max(lowerBound, min(upperBound, affordability + bias))
